@@ -5,9 +5,12 @@
  * Camera capture session
  */
 
+#include "camera_session.h"
+
 #include <iomanip>
 #include <iostream>
 #include <limits.h>
+#include <optional>
 #include <sstream>
 
 #include <libcamera/control_ids.h>
@@ -16,7 +19,6 @@
 #include "../common/event_loop.h"
 #include "../common/stream_options.h"
 
-#include "camera_session.h"
 #include "capture_script.h"
 #include "file_sink.h"
 #ifdef HAVE_KMS
@@ -173,6 +175,11 @@ void CameraSession::listControls() const
 			std::cout << "Control: " << io.str()
 				  << id->vendor() << "::" << id->name() << ":"
 				  << std::endl;
+
+			std::optional<int32_t> def;
+			if (!info.def().isNone())
+				def = info.def().get<int32_t>();
+
 			for (const auto &value : info.values()) {
 				int32_t val = value.get<int32_t>();
 				const auto &it = id->enumerators().find(val);
@@ -182,7 +189,10 @@ void CameraSession::listControls() const
 					std::cout << "UNKNOWN";
 				else
 					std::cout << it->second;
-				std::cout << " (" << val << ")" << std::endl;
+
+				std::cout << " (" << val << ")"
+					  << (val == def ? " [default]" : "")
+					  << std::endl;
 			}
 		}
 
