@@ -6,10 +6,12 @@
  */
 
 #include <algorithm>
+#include <bitset>
 #include <cmath>
 #include <fstream>
 #include <map>
 #include <memory>
+#include <optional>
 #include <set>
 #include <string>
 #include <vector>
@@ -56,6 +58,9 @@ public:
 	Stream stream_;
 	std::map<PixelFormat, std::vector<SizeRange>> formats_;
 
+	std::optional<v4l2_exposure_auto_type> autoExposureMode_;
+	std::optional<v4l2_exposure_auto_type> manualExposureMode_;
+
 private:
 	bool generateId();
 
@@ -93,8 +98,8 @@ public:
 	bool match(DeviceEnumerator *enumerator) override;
 
 private:
-	int processControl(ControlList *controls, unsigned int id,
-			   const ControlValue &value);
+	int processControl(const UVCCameraData *data, ControlList *controls,
+			   unsigned int id, const ControlValue &value);
 	int processControls(UVCCameraData *data, Request *request);
 
 	bool acquireDevice(Camera *camera) override;
@@ -106,6 +111,26 @@ private:
 	}
 };
 
+namespace {
+
+std::optional<controls::ExposureTimeModeEnum> v4l2ToExposureMode(int32_t x)
+{
+	using namespace controls;
+
+	switch (x) {
+	case V4L2_EXPOSURE_AUTO:
+	case V4L2_EXPOSURE_APERTURE_PRIORITY:
+		return ExposureTimeModeAuto;
+	case V4L2_EXPOSURE_MANUAL:
+	case V4L2_EXPOSURE_SHUTTER_PRIORITY:
+		return ExposureTimeModeManual;
+	default:
+		return {};
+	}
+}
+
+} /* namespace */
+
 UVCCameraConfiguration::UVCCameraConfiguration(UVCCameraData *data)
 	: CameraConfiguration(), data_(data)
 {
@@ -287,8 +312,8 @@ void PipelineHandlerUVC::stopDevice(Camera *camera)
 	data->video_->releaseBuffers();
 }
 
-int PipelineHandlerUVC::processControl(ControlList *controls, unsigned int id,
-				       const ControlValue &value)
+int PipelineHandlerUVC::processControl(const UVCCameraData *data, ControlList *controls,
+				       unsigned int id, const ControlValue &value)
 {
 	uint32_t cid;
 
@@ -298,7 +323,7 @@ int PipelineHandlerUVC::processControl(ControlList *controls, unsigned int id,
 		cid = V4L2_CID_CONTRAST;
 	else if (id == controls::Saturation)
 		cid = V4L2_CID_SATURATION;
-	else if (id == controls::AeEnable)
+	else if (id == controls::ExposureTimeMode)
 		cid = V4L2_CID_EXPOSURE_AUTO;
 	else if (id == controls::ExposureTime)
 		cid = V4L2_CID_EXPOSURE_ABSOLUTE;
@@ -332,10 +357,21 @@ int PipelineHandlerUVC::processControl(ControlList *controls, unsigned int id,
 	}
 
 	case V4L2_CID_EXPOSURE_AUTO: {
-		int32_t ivalue = value.get<bool>()
-			       ? V4L2_EXPOSURE_APERTURE_PRIORITY
-			       : V4L2_EXPOSURE_MANUAL;
-		controls->set(V4L2_CID_EXPOSURE_AUTO, ivalue);
+		std::optional<v4l2_exposure_auto_type> mode;
+
+		switch (value.get<int32_t>()) {
+		case controls::ExposureTimeModeAuto:
+			mode = data->autoExposureMode_;
+			break;
+		case controls::ExposureTimeModeManual:
+			mode = data->manualExposureMode_;
+			break;
+		}
+
+		if (!mode)
+			return -EINVAL;
+
+		controls->set(V4L2_CID_EXPOSURE_AUTO, static_cast<int32_t>(*mode));
 		break;
 	}
 
@@ -373,7 +409,7 @@ int PipelineHandlerUVC::processControls(UVCCameraData *data, Request *request)
 	ControlList controls(data->video_->controls());
 
 	for (const auto &[id, value] : request->controls())
-		processControl(&controls, id, value);
+		processControl(data, &controls, id, value);
 
 	for (const auto &ctrl : controls)
 		LOG(UVC, Debug)
@@ -647,7 +683,7 @@ void UVCCameraData::addControl(uint32_t cid, const ControlInfo &v4l2Info,
 		id = &controls::Saturation;
 		break;
 	case V4L2_CID_EXPOSURE_AUTO:
-		id = &controls::AeEnable;
+		id = &controls::ExposureTimeMode;
 		break;
 	case V4L2_CID_EXPOSURE_ABSOLUTE:
 		id = &controls::ExposureTime;
@@ -660,6 +696,7 @@ void UVCCameraData::addControl(uint32_t cid, const ControlInfo &v4l2Info,
 	}
 
 	/* Map the control info. */
+	const std::vector<ControlValue> &v4l2Values = v4l2Info.values();
 	int32_t min = v4l2Info.min().get<int32_t>();
 	int32_t max = v4l2Info.max().get<int32_t>();
 	int32_t def = v4l2Info.def().get<int32_t>();
@@ -697,10 +734,79 @@ void UVCCameraData::addControl(uint32_t cid, const ControlInfo &v4l2Info,
 		};
 		break;
 
-	case V4L2_CID_EXPOSURE_AUTO:
-		info = ControlInfo{ false, true, true };
-		break;
+	case V4L2_CID_EXPOSURE_AUTO: {
+		/*
+		 * From the V4L2_CID_EXPOSURE_AUTO documentation:
+		 *
+		 * ------------------------------------------------------------
+		 * V4L2_EXPOSURE_AUTO:
+		 * Automatic exposure time, automatic iris aperture.
+		 *
+		 * V4L2_EXPOSURE_MANUAL:
+		 * Manual exposure time, manual iris.
+		 *
+		 * V4L2_EXPOSURE_SHUTTER_PRIORITY:
+		 * Manual exposure time, auto iris.
+		 *
+		 * V4L2_EXPOSURE_APERTURE_PRIORITY:
+		 * Auto exposure time, manual iris.
+		 *-------------------------------------------------------------
+		 *
+		 * ExposureTimeModeAuto = { V4L2_EXPOSURE_AUTO,
+		 * 			    V4L2_EXPOSURE_APERTURE_PRIORITY }
+		 *
+		 *
+		 * ExposureTimeModeManual = { V4L2_EXPOSURE_MANUAL,
+		 *			      V4L2_EXPOSURE_SHUTTER_PRIORITY }
+		 */
 
+		std::bitset<
+			std::max(V4L2_EXPOSURE_AUTO,
+			std::max(V4L2_EXPOSURE_APERTURE_PRIORITY,
+			std::max(V4L2_EXPOSURE_MANUAL,
+				 V4L2_EXPOSURE_SHUTTER_PRIORITY))) + 1
+		> exposureModes;
+		std::optional<controls::ExposureTimeModeEnum> lcDef;
+
+		for (const ControlValue &value : v4l2Values) {
+			const auto x = value.get<int32_t>();
+
+			if (0 <= x && static_cast<std::size_t>(x) < exposureModes.size()) {
+				exposureModes[x] = true;
+
+				if (x == def)
+					lcDef = v4l2ToExposureMode(x);
+			}
+		}
+
+		if (exposureModes[V4L2_EXPOSURE_AUTO])
+			autoExposureMode_ = V4L2_EXPOSURE_AUTO;
+		else if (exposureModes[V4L2_EXPOSURE_APERTURE_PRIORITY])
+			autoExposureMode_ = V4L2_EXPOSURE_APERTURE_PRIORITY;
+
+		if (exposureModes[V4L2_EXPOSURE_SHUTTER_PRIORITY])
+			manualExposureMode_ = V4L2_EXPOSURE_SHUTTER_PRIORITY;
+		else if (exposureModes[V4L2_EXPOSURE_MANUAL])
+			manualExposureMode_ = V4L2_EXPOSURE_MANUAL;
+
+		std::array<ControlValue, 2> values;
+		std::size_t count = 0;
+
+		if (autoExposureMode_)
+			values[count++] = controls::ExposureTimeModeAuto;
+
+		if (manualExposureMode_)
+			values[count++] = controls::ExposureTimeModeManual;
+
+		if (count == 0)
+			return;
+
+		info = ControlInfo{
+			Span<const ControlValue>{ values.data(), count },
+			!lcDef ? values.front() : *lcDef,
+		};
+		break;
+	}
 	case V4L2_CID_EXPOSURE_ABSOLUTE:
 		/*
 		 * ExposureTime is in units of 1 µs, and UVC expects
